@@ -26,6 +26,7 @@
 
 #include <errno.h>
 #include <string.h>
+#include <stdio.h>
 #include <sys/stat.h>
 
 #ifdef HAVE_DIRENT_H
@@ -79,6 +80,7 @@ g_dir_open (const gchar  *path,
             GError      **error)
 {
   GDir *dir;
+  int errsv;
 #ifdef G_OS_WIN32
   wchar_t *wpath;
 #else
@@ -102,12 +104,13 @@ g_dir_open (const gchar  *path,
     return dir;
 
   /* error case */
+  errsv = errno;
 
   g_set_error (error,
 	       G_FILE_ERROR,
-	       g_file_error_from_errno (errno),
+	       g_file_error_from_errno (errsv),
 	       _("Error opening directory '%s': %s"),
-	       path, g_strerror (errno));
+	       path, g_strerror (errsv));
   
   g_free (dir);
       
@@ -121,13 +124,16 @@ g_dir_open (const gchar  *path,
     return dir;
 
   /* error case */
+  errsv = errno;
+
   utf8_path = g_filename_to_utf8 (path, -1,
 				  NULL, NULL, NULL);
+
   g_set_error (error,
                G_FILE_ERROR,
-               g_file_error_from_errno (errno),
+               g_file_error_from_errno (errsv),
                _("Error opening directory '%s': %s"),
-	       utf8_path, g_strerror (errno));
+	       utf8_path, g_strerror (errsv));
 
   g_free (utf8_path);
   g_free (dir);
@@ -136,7 +142,7 @@ g_dir_open (const gchar  *path,
 #endif
 }
 
-#ifdef G_OS_WIN32
+#if defined (G_OS_WIN32) && !defined (_WIN64)
 
 /* The above function actually is called g_dir_open_utf8, and it's
  * that what applications compiled with this GLib version will
@@ -226,7 +232,7 @@ g_dir_read_name (GDir *dir)
 #endif
 }
 
-#ifdef G_OS_WIN32
+#if defined (G_OS_WIN32) && !defined (_WIN64)
 
 /* Ditto for g_dir_read_name */
 
